#!/usr/bin/perl

## Nobumi Iyanaga 2005.10.11

use strict;
use warnings;

use File::Find;
use Encode;
use utf8;

# @ARGV = ("/Volumes/myDocument/Users/ni/Desktop/t01/", "/Volumes/myDocument/Users/ni/Desktop/t01_utf8/", "Shift_JIS", "all", 1, "html");

my $source_folder = shift;
my $dest_folder = shift;
my $source_encoding = shift;

my $file_filter = shift; # grep pattern, like "\.txt", "\.txt|\.html", etc. Can be "all" too.

my $eol_conversion = shift; # Unix end of lines: 0 or 1

my $further_conversion = shift; # conversion type name -- e.g. "html_entity", "html", or 0

if ($file_filter && $file_filter =~ /[^\\]?\.[^\\\|\.]+/) {
	$file_filter =~ s/\./\\\./g;
}
elsif (! $file_filter) {
	$file_filter = '\.txt';
}

if ($source_folder !~ /^.+\/$/) {
	$source_folder .= "/";
}

if ($dest_folder !~ /^.+\/$/) {
	$dest_folder .= "/";
}

$source_encoding = uc ($source_encoding);

#	 iconv -l | sort result:
#	 850 CP850 IBM850 CSPC850MULTILINGUAL
#	 862 CP862 IBM862 CSPC862LATINHEBREW
#	 866 CP866 IBM866 CSIBM866
#	 ANSI_X3.4-1968 ANSI_X3.4-1986 ASCII CP367 IBM367 ISO-IR-6 ISO646-US ISO_646.IRV:1991 US US-ASCII CSASCII
#	 ARABIC ASMO-708 ECMA-114 ISO-8859-6 ISO-IR-127 ISO8859-6 ISO_8859-6 ISO_8859-6:1987 CSISOLATINARABIC
#	 ARMSCII-8
#	 BIG-5 BIG-FIVE BIG5 BIGFIVE CN-BIG5 CSBIG5
#	 BIG5-HKSCS BIG5HKSCS
#	 C99
#	 CHINESE GB_2312-80 ISO-IR-58 CSISO58GB231280
#	 CN GB_1988-80 ISO-IR-57 ISO646-CN CSISO57GB1988
#	 CN-GB EUC-CN EUCCN GB2312 CSGB2312
#	 CN-GB-ISOIR165 ISO-IR-165
#	 CP1133 IBM-CP1133
#	 CP1250 MS-EE WINDOWS-1250
#	 CP1251 MS-CYRL WINDOWS-1251
#	 CP1252 MS-ANSI WINDOWS-1252
#	 CP1253 MS-GREEK WINDOWS-1253
#	 CP1254 MS-TURK WINDOWS-1254
#	 CP1255 MS-HEBR WINDOWS-1255
#	 CP1256 MS-ARAB WINDOWS-1256
#	 CP1257 WINBALTRIM WINDOWS-1257
#	 CP1258 WINDOWS-1258
#	 CP1361 JOHAB
#	 CP819 IBM819 ISO-8859-1 ISO-IR-100 ISO8859-1 ISO_8859-1 ISO_8859-1:1987 L1 LATIN1 CSISOLATIN1
#	 CP874 WINDOWS-874
#	 CP932
#	 CP936 GBK MS936 WINDOWS-936
#	 CP949 UHC
#	 CP950
#	 CYRILLIC ISO-8859-5 ISO-IR-144 ISO8859-5 ISO_8859-5 ISO_8859-5:1988 CSISOLATINCYRILLIC
#	 ECMA-118 ELOT_928 GREEK GREEK8 ISO-8859-7 ISO-IR-126 ISO8859-7 ISO_8859-7 ISO_8859-7:1987 CSISOLATINGREEK
#	 EUC-JP EUCJP EXTENDED_UNIX_CODE_PACKED_FORMAT_FOR_JAPANESE CSEUCPKDFMTJAPANESE
#	 EUC-KR EUCKR CSEUCKR
#	 EUC-TW EUCTW CSEUCTW
#	 GB18030
#	 GEORGIAN-ACADEMY
#	 GEORGIAN-PS
#	 HEBREW ISO-8859-8 ISO-IR-138 ISO8859-8 ISO_8859-8 ISO_8859-8:1988 CSISOLATINHEBREW
#	 HP-ROMAN8 R8 ROMAN8 CSHPROMAN8
#	 HZ HZ-GB-2312
#	 ISO-10646-UCS-2 UCS-2 CSUNICODE
#	 ISO-10646-UCS-4 UCS-4 CSUCS4
#	 ISO-2022-CN CSISO2022CN
#	 ISO-2022-CN-EXT
#	 ISO-2022-JP CSISO2022JP
#	 ISO-2022-JP-1
#	 ISO-2022-JP-2 CSISO2022JP2
#	 ISO-2022-KR CSISO2022KR
#	 ISO-8859-10 ISO-IR-157 ISO8859-10 ISO_8859-10 ISO_8859-10:1992 L6 LATIN6 CSISOLATIN6
#	 ISO-8859-13 ISO-IR-179 ISO8859-13 ISO_8859-13 L7 LATIN7
#	 ISO-8859-14 ISO-CELTIC ISO-IR-199 ISO8859-14 ISO_8859-14 ISO_8859-14:1998 L8 LATIN8
#	 ISO-8859-15 ISO-IR-203 ISO8859-15 ISO_8859-15 ISO_8859-15:1998 LATIN-9
#	 ISO-8859-16 ISO-IR-226 ISO8859-16 ISO_8859-16 ISO_8859-16:2001 L10 LATIN10
#	 ISO-8859-2 ISO-IR-101 ISO8859-2 ISO_8859-2 ISO_8859-2:1987 L2 LATIN2 CSISOLATIN2
#	 ISO-8859-3 ISO-IR-109 ISO8859-3 ISO_8859-3 ISO_8859-3:1988 L3 LATIN3 CSISOLATIN3
#	 ISO-8859-4 ISO-IR-110 ISO8859-4 ISO_8859-4 ISO_8859-4:1988 L4 LATIN4 CSISOLATIN4
#	 ISO-8859-9 ISO-IR-148 ISO8859-9 ISO_8859-9 ISO_8859-9:1989 L5 LATIN5 CSISOLATIN5
#	 ISO-IR-14 ISO646-JP JIS_C6220-1969-RO JP CSISO14JISC6220RO
#	 ISO-IR-149 KOREAN KSC_5601 KS_C_5601-1987 KS_C_5601-1989 CSKSC56011987
#	 ISO-IR-159 JIS_X0212 JIS_X0212-1990 JIS_X0212.1990-0 X0212 CSISO159JISX02121990
#	 ISO-IR-166 TIS-620 TIS620 TIS620-0 TIS620.2529-1 TIS620.2533-0 TIS620.2533-1
#	 ISO-IR-87 JIS0208 JIS_C6226-1983 JIS_X0208 JIS_X0208-1983 JIS_X0208-1990 X0208 CSISO87JISX0208
#	 JAVA
#	 JISX0201-1976 JIS_X0201 X0201 CSHALFWIDTHKATAKANA
#	 KOI8-R CSKOI8R
#	 KOI8-RU
#	 KOI8-T
#	 KOI8-U
#	 MAC MACINTOSH MACROMAN CSMACINTOSH
#	 MACARABIC
#	 MACCENTRALEUROPE
#	 MACCROATIAN
#	 MACCYRILLIC
#	 MACGREEK
#	 MACHEBREW
#	 MACICELAND
#	 MACROMANIA
#	 MACTHAI
#	 MACTURKISH
#	 MACUKRAINE
#	 MS_KANJI SHIFT-JIS SHIFT_JIS SJIS CSSHIFTJIS
#	 MULELAO-1
#	 NEXTSTEP
#	 TCVN TCVN-5712 TCVN5712-1 TCVN5712-1:1993
#	 UCS-2-INTERNAL
#	 UCS-2-SWAPPED
#	 UCS-2BE UNICODE-1-1 UNICODEBIG CSUNICODE11
#	 UCS-2LE UNICODELITTLE
#	 UCS-4-INTERNAL
#	 UCS-4-SWAPPED
#	 UCS-4BE
#	 UCS-4LE
#	 UNICODE-1-1-UTF-7 UTF-7 CSUNICODE11UTF7
#	 UTF-16
#	 UTF-16BE
#	 UTF-16LE
#	 UTF-32
#	 UTF-32BE
#	 UTF-32LE
#	 UTF-8
#	 VISCII VISCII1.1-1 CSVISCII


if ($source_encoding eq "BIG5-ETEN") {
	$source_encoding = "CP950";
}

my $iconv_command = "iconv -s -f " . $source_encoding . " -t UTF-8 ";

my @files;

if (! -d $source_folder) {
	die ("The source folder $source_folder could not be found...: $!");
}

if (! -e $dest_folder) {
	mkdir ($dest_folder);
}
my %entity_data;

find (\&recursive_process, $source_folder);

foreach my $file (@files) {
	my ($parent_folder, $fname) = $file =~ /^(.+\/)(.+)/;
	my $temp_source_folder = quotemeta ($source_folder);
	my $temp_dest_folder = quotemeta ($dest_folder);

	$parent_folder =~ s/$temp_source_folder/$temp_dest_folder/;
	$parent_folder =~ s/\\//g;
	if (! -e $parent_folder) {
		`mkdir -p "$parent_folder"`;
	}
	my $dest_file = $parent_folder . $fname;
	
	$iconv_command .= "'" . $file . "'";
	if (! $further_conversion) {
		if (-e $dest_file) {
	#		die ("A dest file $dest_file already exists...");
			print "A dest file $dest_file already exists...";
			exit;
		}
		
		$iconv_command .= " > '" . $dest_file . "'";
		print $iconv_command, "\n";
		`$iconv_command`;
		if ($eol_conversion) {
			my $conv_script = q (/usr/bin/perl -pi -ne 's/\r\n?/\n/g' ") . $dest_file . '"';
			`$conv_script`;
#			print $conv_script, "\n";
		}
		$iconv_command = "iconv -s -f " . $source_encoding . " -t UTF-8 ";
	}
	else { # the "further_conversion" is defined...
		my $temp_str = `$iconv_command`;
		
		$temp_str = decode ('utf8', $temp_str);
		$temp_str = &further_conversion ($temp_str, $further_conversion, $fname);
				
		if ($eol_conversion) {
			$temp_str =~ s/\r\n?/\n/g;
		}
		
		if ($further_conversion eq "html") {
			my ($temp_fname, $ext) = $fname =~ /^(.+)(\..+)$/;
			if (! $ext) {
				$dest_file .= ".html";
			}
			else {
				$dest_file = $parent_folder . $temp_fname . ".html";
			}
# 			print $dest_file, "\n";
			if (-e $dest_file) {
				print "A dest file $dest_file already exists...";
				exit;
			}
		}
		
		open (OUT, '>:utf8', $dest_file);
		
		print OUT $temp_str;
		
		close (OUT);
		$iconv_command = "iconv -s -f " . $source_encoding . " -t UTF-8 ";
		
	}
}

###############

sub recursive_process {

	my $file = $File::Find::name;
	push( @files, $file ) if( -f $file && &file_filter ($file) );

}

sub file_filter {
	my $file = shift;
	
	if ($file_filter =~ /^all$/i) {
		if (-T $file) {
			return 1;
		}
		else {
			return 0;
		}
	}
	
	if ($file =~ /$file_filter$/i) {
		return 1;
	}
	else {
		return 0;
	}
	
}

sub further_conversion {
	my $str = shift;
	my $conversion_type = shift;
	my $fname = shift;
	
	if ($conversion_type eq "html_entity") {
		if (! %entity_data) {
			while (<DATA>) {
				chomp;
				my ($entity, $value) = split (/\t/, $_);
				$entity_data{$entity} = $value;
			}
		}
		$str =~ s/\&(.+?);/&html_entity ($1)/sge;
		
	}
	elsif ($conversion_type eq "html") {
		my $htmlHeader1 = "<html>\n<head>\n<title>";
		$htmlHeader1 .= $fname;

		my $htmlHeader2 = "</title>\n<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">\n</head>\n<body alink=\"#008000\" bgcolor=\"#FFFFFF\" link=\"#0000FF\" vlink=\"#800080\">\n";

		my $htmlFooter = "</body>\n</html>\n";
		
		$str =~ s/</\&gt;/g;
		$str =~ s/</\&lt;/g;
		$str =~ s/\t/\&nbsp;\&nbsp;\&nbsp;\&nbsp;/g;
		$str = &brp_tag ($str);
		
		$str = $htmlHeader1 . $htmlHeader2 . $str . $htmlFooter;
		
	}
	
	return ($str);	
}

sub brp_tag {
	my $text_here = shift;

	$text_here =~ s/(\n\n)/<p>$1/sg;
	$text_here =~ s/(\n[^\n].)/<br>$1/sg;
	$text_here =~ s/\n<br>\n/\n\n/sg;
	$text_here =~ s/<\/?p><br>\n/<\/p>\n/sg;

	return ($text_here);
}

sub html_entity {
	my $entity = shift;
	
	if (exists $entity_data{$entity}) {
		return (pack ("U*", $entity_data{$entity}));
	}
	elsif ($entity =~ /^#\d+$/) {
		$entity =~ s/^#//;
		return (pack ("U*", $entity));
	}
	elsif ($entity =~ /^#x[0-9A-Fa-f]+$/) {
		$entity =~ s/^#x//;
		return (pack ("U*", hex ($entity)));
	}
	else {
		return ($entity);
	}
	
}
	
	
__END__
nbsp	160
iexcl	161
cent	162
pound	163
curren	164
yen	165
brvbar	166
sect	167
uml	168
copy	169
ordf	170
laquo	171
not	172
shy	173
reg	174
macr	175
deg	176
plusmn	177
sup2	178
sup3	179
acute	180
micro	181
para	182
middot	183
cedil	184
sup1	185
ordm	186
raquo	187
frac14	188
frac12	189
frac34	190
iquest	191
Agrave	192
Aacute	193
Acirc	194
Atilde	195
Auml	196
Aring	197
AElig	198
Ccedil	199
Egrave	200
Eacute	201
Ecirc	202
Euml	203
Igrave	204
Iacute	205
Icirc	206
Iuml	207
ETH	208
Ntilde	209
Ograve	210
Oacute	211
Ocirc	212
Otilde	213
Ouml	214
times	215
Oslash	216
Ugrave	217
Uacute	218
Ucirc	219
Uuml	220
Yacute	221
THORN	222
szlig	223
agrave	224
aacute	225
acirc	226
atilde	227
auml	228
aring	229
aelig	230
ccedil	231
egrave	232
eacute	233
ecirc	234
euml	235
igrave	236
iacute	237
icirc	238
iuml	239
eth	240
ntilde	241
ograve	242
oacute	243
ocirc	244
otilde	245
ouml	246
divide	247
oslash	248
ugrave	249
uacute	250
ucirc	251
uuml	252
yacute	253
thorn	254
yuml	255
